<?php

if (!defined('_JOOMLMS_FRONT_HOME')) { define('_JOOMLMS_FRONT_HOME', JPATH_SITE .DS. 'components' .DS. 'com_joomla_lms'); }
if (!defined('_JLMS_EXEC')) { define('_JLMS_EXEC', true); }


require_once (_JOOMLMS_FRONT_HOME . DS . 'libraries' .  DS . 'servicelocator.php');
\LMS\Libraries\ServiceLocator::registerNamespace('LMS', _JOOMLMS_FRONT_HOME);
\LMS\Libraries\ServiceLocator::registerNamespace('Gtm', __DIR__);
\LMS\Libraries\ServiceLocator::activate();

use LMS\Tools\Conference\Base;
use LMS\Libraries\Http;
use LMS\Libraries\Exception\Operation;
use LMS\Tools\Conference;
use LMS\Libraries\Exception\Database;

use Gtm\Service;
use Gtm\Repository;

/**
 * Plugin that provides functionality for build-in Joomla LMS conferencing
 *
 * @property Service\Response\Processor responseService
 */
class plgSystemGtmConference extends Base\Plugin
{
    private $oauthData;
	private $httpRequest;

    public function __construct( &$subject, $config )
    {
        $this->dir = __DIR__ .'/templates/';
        parent::__construct( $subject, $config );
		$this->httpRequest = new Http\Request();
    }

    protected function initServices($diContainer)
    {
        parent::initServices($diContainer);

        $this->responseService = new Service\Response\Processor($diContainer);
		$this->bookingService = new Gtm\Service\Booking(array(
			'container' => $diContainer,
			'repository' => $this->getConferenceBookingRepository(\LMS\Libraries\ServiceLocator::get('db'))));

    }


    public function onOAuth($diContainer)
    {
        $this->diContainer = $diContainer;
        $this->eventParams = $diContainer['registry']->get('params');

        $responseKey = $this->eventParams['code'];
        $apikey = $this->params->get('apikey');

        $headers = array(
            "Accept: application/json",
            "Content-type: application/json",
        );

        $response = $this->httpRequest->sendJSONRequest('https://api.citrixonline.com/oauth/access_token?grant_type=authorization_code&code='.$responseKey.'&client_id='.$apikey, array(), 'GET', $headers);

        $url = sefRelToAbs("../../index.php?option=".$this->eventParams['option']."&view=default");
        if($response === false){
			throw new Operation\Error(
				'Error sending request to GTM service',
				sefRelToAbs($url)
			);
        }

        $data = json_decode($response, true);
        $data['user_id'] = $this->params->get('isMultiple') ? $this->eventParams['my']->id : 0;

        $model = $this->getConferenceOAuthRepository($this->diContainer['db']);

        if(!$model->save($data)){
			throw new Operation\Error(
				'Error while saving OAuth data',
				sefRelToAbs($url)
			);
		}

		if($this->params->get('isMultiple')){
			$dataToUpdate = array(
				'token' => $data['access_token'],
				'refresh_token' => $data['refresh_token'],
				'created' => $this->dateHandler->getCurrentUTCDatetime()
			);

			if(!$model->update($dataToUpdate, array('organizer_key' => $data['organizer_key']))){
				throw new Operation\Error(
					'Error while updating OAuth data',
					sefRelToAbs($url)
				);
			}
		}

        return $this->responseService->operationOk('Authorization has been successfully completed');
    }




    public function onConferenceEnterRoom($diContainer)
    {
        $response = $this->httpRequest->sendJSONRequest('https://api.citrixonline.com/G2M/rest/meetings/'.$this->eventParams['cid'].'/start', array(), 'GET', $this->httpRequest->getHeadersForGTM($this->oauthData->token));
        if($response === false){
            throw new Operation\Error(
				'Error sending response to GoToMeeting server',
				sefRelToAbs('index.php?option=com_joomla_lms&task=conference&Itemid='.$this->eventParams['Itemid'].'&id='.$this->eventParams['courseID'])
			);
        }

        $responseObject = json_decode($response);
        $this->responseService->addRenderParams(array(
            'hostURL' => $responseObject->hostURL
        ));
        return parent::onConferenceEnterRoom($diContainer);
    }

    public function onConferenceSynchronize($diContainer)
    {
        if(!$this->bookingService->synchronizeBookedConferences($this->oauthData)){
			throw new Operation\Error('Error occurred while synchronizing data');
		}
        return $this->responseService->operationOk('Synchronization has been successfully completed');
    }

    public function onConferenceNewRecord($diContainer)
    {
        $lists = array();
        $lists = array_merge($lists, $this->viewService->prepareYesNoLists(array('published' => 0)));
        $lists = array_merge($lists, $this->viewService->getTimeList());

        $this->responseService->addRenderParams(array(
            'lists' => $lists,
        ));
        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderNewRecord();
    }

    public function onConferenceCreateRecord($diContainer)
    {
        $this->diContainer = $diContainer ;

        $this->eventParams['session_name'] = md5(time());


		$dateStamp = $this->dateHandler->getTimestamp($this->eventParams['start_date'], $this->dateHandler->getTimezoneName());
		$date = $this->dateHandler->getDatetime('Y-m-d', $dateStamp);

        $this->eventParams['start_date'] = $this->dateHandler->getUTCDatetime('Y-m-d H:i:s', $this->viewService->prepareDate($this->eventParams['time'], $this->eventParams['minutes'], $date));
        $this->eventParams['user_id'] = $this->eventParams['my']->id;

        $recordFile = $this->fileService->uploadRecordFile('record_file');
		if(!$recordFile){
			throw new Operation\Error(
				'Record file has not been loaded',
				$this->responseService->getNewRecordRedirectURL($this->eventParams)
			);
		}

        $repository = $this->getConferenceRecordRepository($diContainer['db']);

		if(!$repository->save($this->eventParams)){
			throw new Database\Error('Error while saving record');
		}

		if(!$repository->saveFile($repository->lastInsertedID(), $recordFile)){
			throw new Operation\Error(
				'Error while saving record',
				$this->responseService->getNewRecordRedirectURL($this->eventParams));
		}

        return $this->responseService->operationOk('Record has been successfully created');
    }

    public function onConferencePlayback($diContainer)
    {
        $model = $this->getConferenceRecordRepository($this->diContainer['db']);
        $record = $model->getBySessionName($this->eventParams['session_name']);

        if(is_null($record)){
            throw new Operation\Error(
				'There is no such record',
				sefRelToAbs('index.php?option=com_joomla_lms&Itemid='.$this->eventParams['Itemid'].'&task=conference.archive&id='.$this->eventParams['courseID'])
			);
        }

        return $this->responseService->file($record->filename, $this->responseService->getHeadersForDownloadFile($record->filename));
    }





    /*********************************************************************************
     *
     *
     *                              PRIVATE METHODS
     *
     *
     ********************************************************************************/

    /**
     * @param $diContainer
     */
    protected function beforeEventCalled($diContainer)
    {
        parent::beforeEventCalled($diContainer);

		$diContainer['registry']->set('apikey', $this->eventParams['apikey']);

		$eventParams = $diContainer['registry']->get('params');

		$oauthService = new Service\Oauth(array(
			'repository' => $this->getConferenceOAuthRepository($diContainer['db']),
			'container' => $diContainer
		));

		$oauthService->checkAuthorization($eventParams['my']->id, $this->eventParams['isManager'], $this->params->get('isMultiple'));
		$this->oauthData = $oauthService->getOAuthData();
    }

    /**
     * @param $diContainer
     * @return array
     */
    protected function _getParamsStartPage($params)
    {
        $model = $this->getConferenceBookingRepository($this->diContainer['db']);
        $conferences = $model->getAvailable($params['courseID'], $this->diContainer['config']->get('offset'), true);
        return array(
            'session' => $this->diContainer['session'],
            'ItemID' => $params['ItemID'],
            'option' => $params['option'],
            'courseID' => $params['courseID'],
            'userType' => $params['usertype'],
            'conferences' => $conferences
        );
    }

    /**
     * @param $diContainer
     * @return array
     */
    protected function _getParamsEnterRoom($params)
    {
        $config = $this->diContainer['config'];
        return array(
            'session' => $params['session'],
            'liveSite' => $config->get('live_site'),
            'usertype' => $params['usertype'],
            'recordedSession' => $params['recordedSession'],
            'ItemID' =>$params['ItemID'],
            'courseID' => $params['courseID'],
        );
    }



    protected function getConferenceBookingRepository($db)
    {
        return new Repository\Booking(array('db' => $db, 'oauthData' => $this->oauthData));
    }

    protected function getConferenceRecordRepository($db)
    {
        return new Repository\Record(array('db' => $db, 'oauthData' => $this->oauthData));
    }

    protected function getConferenceOAuthRepository($db)
    {
        return new Conference\Repositories\Oauth(array('db' => $db, 'oauthData' => $this->oauthData));
    }

}

