<?php

jimport('joomla.plugin.plugin');

if (!defined('_JOOMLMS_FRONT_HOME')) { define('_JOOMLMS_FRONT_HOME', JPATH_SITE .DS. 'components' .DS. 'com_joomla_lms'); }
if (!defined('_JLMS_EXEC')) { define('_JLMS_EXEC', true); }

require_once (_JOOMLMS_FRONT_HOME . DS . 'libraries' .  DS . 'servicelocator.php');
\LMS\Libraries\ServiceLocator::registerNamespace('LMS', _JOOMLMS_FRONT_HOME);
\LMS\Libraries\ServiceLocator::registerNamespace('Jlms', __DIR__);
\LMS\Libraries\ServiceLocator::activate();

use LMS\Tools\Conference\Base;
use LMS\Libraries\Injection;
use LMS\Tools\Conference\Repository;

/**
 * Plugin that provides functionality for build-in Joomla LMS conferencing
 *
 * @property \Jlms\Service\Response\Processor $responseService
 *
 */
class plgSystemJlmsConference extends Base\Plugin
{

    public function __construct( &$subject, $config )
    {
        $this->dir = __DIR__ .'/templates/';
        parent::__construct( $subject, $config );
    }


    protected function initServices($diContainer)
    {
        parent::initServices($diContainer);

        $this->responseService = new \Jlms\Service\Response\Processor($diContainer);

    }



    /**
     * Custom event that triggered when try to access profile
     *
     * @param Injection\Container $this->eventParams
     * @param string $diContainer['username']
     */
    public function onConferenceProfile($diContainer)
    {
        $model = $this->getConferenceUserModel($diContainer['db']);

        $this->responseService->addRenderParams(array(
            'userData' => $model->getInfo($this->eventParams['username'])
        ));
        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderProfile();
    }

    /**
     * Custom event that triggered when try to access cb profile
     *
     * @param mixed $db
     * @param mixed $config
     * @param string $diContainer['username']
     */
    public function onConferenceConferenceProfileCb($diContainer)
    {
        $model = new Repository\User($diContainer);
        $userData = $model->getInfo($this->eventParams['username']);

        return $this->responseService->rawDataOk($userData->id);
    }

    /**
     * Custom event that triggered when requested params. It doesn't return anything - just generate XML Response
     *
     * @param Injection\Container $diContainer
     * @param mixed $diContainer['my']
     */
    public function onConferenceParamRequest($diContainer)
    {
        $isAvailableConference = $this->bookingService->isAvailableConferenceExists($this->eventParams['courseID'], $this->eventParams['my']);

        $data = $this->responseService->generateBookingParamXMLResponse($isAvailableConference);
        return $this->responseService->xmlOk($data);
    }

    /**
     * Custom event that triggered when requested params. It doesn't return anything - just generate XML Response
     *
     * @param Injection\Container $diContainer
     * @param mixed $diContainer['my']
     */
    public function onConferenceParams($diContainer)
    {
        $pseudo = $this->eventParams['my']->username;
        $acl = JLMSFactory::getACL();
        $iso = explode( '=', _ISO );

        $courseModel = $this->getCourseRepository($this->diContainer['db']);
        $files = $courseModel->getFiles($this->eventParams['courseID']);

        $data = $this->responseService->generateParamsXMLResponse($diContainer['config'], $this->params, $iso, $pseudo, $acl, $files);
        return $this->responseService->xmlOk($data);
    }

    /**
     * Custom event that triggered when file uploads
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceUpload($diContainer)
    {
        $config = $diContainer['config'];
        $courseID = $this->eventParams['courseID'];

        $fileID = $this->fileService->uploadForCourse($courseID, 'fichier');

        $docModel = $this->getConferenceDocModel($this->diContainer['db']);

        $this->eventParams['user_id'] = $this->eventParams['my']->id;
        $this->eventParams['filename'] = $_FILES['fichier']['name'];
        $this->eventParams['file_id'] = $fileID;
        $this->eventParams['file_id'] = $fileID;

        if (($docModel->save($this->eventParams))){
            $this->responseService->addRenderParams(array(
                'fileID' => $fileID,
                'courseID' => $courseID,
                'liveSite' => $config->get('live_site'),
                'uploadedFilename' => $_FILES['fichier']['name']
            ));
            return $this->responseService->renderUpload();
        }

        return $this->responseService->jsonError('Error while uploading file');
    }

    /**
     * Custom event that triggered when upload pop up form is requested
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceUploadPopup($diContainer)
    {
        $config = $diContainer['config'];

        $doc = JFactory::getDocument(); $doc->addStylesheet(JUri::base() . '/components/com_joomla_lms/tools/conference/css/conference.css');
        $doc->setTitle('Upload a file');

        $this->responseService->addRenderParams(array(
            'ItemID' => $this->eventParams['ItemID'],
            'option' => $this->eventParams['option'],
            'myID' => $this->eventParams['my']->id,
            'courseID' => $this->eventParams['courseID'],
            'liveSite' => $config->get('live_site')
        ));
        return $this->responseService->renderUploadPopUp();
    }

    /**
     * Custom event that triggered when request to delete uploaded file from conference
     *
     * @param Injection\Container $diContainer
     * @return string
     */
    public function onConferenceDeleteFile($diContainer)
    {
        $courseID = $this->eventParams['courseID'];
        $fileID = $this->eventParams['file_id'];

        $docModel = $this->getConferenceDocModel($diContainer['db']);
        $file = $docModel->getCourseDocumentByID($courseID, $fileID);

        if (is_object($file)) {
            if($this->fileService->isCorrectUploaded($file)){
                $docModel->deleteFileFromLMS($file->file_id);
            }
            $this->fileService->unlink($file);
        }

        $isSuccessfullyDeleted = $docModel->deleteFileFromConference($courseID, $fileID);
        return $isSuccessfullyDeleted ?
            $this->responseService->jsonOk(array('status'=>'OK')) :
            $this->responseService->jsonError(array('status'=>'Error')) ;
    }

    /**
     * @param $diContainer
     * @return bool
     */
    public function onConferenceDisplayFile($diContainer)
    {
        $fileID = $this->eventParams['id'];
        $courseID = $this->eventParams['courseID'];
        $config = $diContainer['config'];

        $docModel = $this->getConferenceDocModel($this->diContainer['db']);
        $file = ($fileID == -1) ?
            $this->fileService->getIntroFile() :
            $docModel->getCourseDocumentByID($courseID, $fileID);

        if(!is_object($file)){
            return $this->responseService->jsonError('File does not exist');
        }

        if($file->upload_type == 1){
            return $this->responseService->LMSfile($file->file_id);
        }

        $fileName = $file->filename;
        $srvName = $config->get('absolute_path')."/components/com_joomla_lms/upload/".$fileName;

        if ( !file_exists( $srvName ) || !is_readable( $srvName ) ){
            return $this->responseService->jsonError('File does not exist or is not readable');
        }

        $userBrowser = $this->responseService->detectUserBrowser($this->eventParams['agent']);
        $mimeType = $this->responseService->getFileMimeType($userBrowser);
        $headers = $this->responseService->getHeadersForDisplayingFile($userBrowser, $mimeType, $fileName, filesize($srvName));

        return $this->responseService->file($srvName, $headers);
    }

    /**
     * @param $diContainer
     * @return string
     */
    public function onConferencePlayback($diContainer)
    {
        $config = $diContainer['config'];

        $this->responseService->addRenderParams(array(
                'streamName' => $this->eventParams['record_name'],
                'is_cb_installed' => $config->get('is_cb_installed'),
                'globBg' => $this->params->get('conf_background'),
                'recordedSession' => $this->eventParams['session_name'],
                'maxClients' => $this->params->get('maxConfClients'),
                'liveSite' => $config->get('live_site'),
        ));
        $this->responseService->addRenderParams($this->eventParams);
        return $this->responseService->renderConferencePlayback();
    }


    /*********************************************************************************
     *
     *
     *                              PRIVATE METHODS
     *
     *
     ********************************************************************************/


    protected function _getParamsStartPage($params)
    {

        $config = $this->diContainer['config'];
        return array(
            'flashComRoot' => $this->params->get('flascommRoot'),
            'maxClients' => $this->params->get('maxConfClients'),
            'liveSite' => $config->get('live_site'),
            'conferenceDescription' => $this->params->get('conf_description'),
            'ItemID' => $params['ItemID'],
            'option' => $params['option'],
            'courseID' => $params['courseID'],
            'enableBooking' => $params['enableBooking'],
            'userType' => $params['usertype'],
            'roomHeight' => 643,
            'recordedSession' => isset($params['recordedSession']) ? $params['recordedSession'] : ''
        );
    }

    protected function _getParamsEnterRoom($params)
    {
        $config = $this->diContainer['config'];
        return array(
            'liveSite' => $config->get('live_site'),
            'usertype' => $params['usertype'],
            'recordedSession' => isset($params['recordedSession']) ? $params['recordedSession'] : '',
            'ItemID' => $params['ItemID'],
            'courseID' => $params['courseID'],
            'uploadHeight' => 75,
            'globBg' => $this->params->get('conf_background'),
            'isCbInstalled' => $config->get('is_cb_installed')
        );
    }

}
