<?php
/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 *
 */

namespace LMS\Tools\Conference\Controllers;

defined('_JEXEC') or die;

require __DIR__ .'/conference.php';

/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 * @since       3.1
 */
class Booking extends Conference
{

	public function __construct($config=array())
	{
		parent::__construct($config);
		$this->input->set('view', 'booking');
	}


	/**
	 * Method to display a view.
	 *
	 * @param   boolean			If true, the view output will be cached
	 * @param   array  An array of safe url parameters and their variable types, for valid values see {@link JFilterInput::clean()}.
	 *
	 * @return  JController		This object to support chaining.
	 * @since   1.5
	 */
	public function display($cachable = false, $urlparams = false)
	{

		if(!$this->acl->CheckPermissions('conference', 'manage')){
			throw new \LMS\Libraries\Exception\Access\Forbidden(_JLMS_ERROR_ACCESS_FORBIDDEN);
		}

		$limit = $this->input->getInt('limit', $this->session->get('list_limit', $this->config->get('list_limit')));
		$this->session->set('list_limit', $limit);
		$limitStart = $this->input->getInt('limitstart', $this->session->get('limitstart_confbook',-99));

		$doRequery = false;
		if ($limitStart == -99) {
			$limitStart = 0;
			$doRequery = true;
		}

		$this->session->set('limitstart_confbook', $limitStart);

		$filterStudent	= intval( mosGetParam( $_REQUEST, 'filter_stu', $this->session->get('filter_stu',0) ) );
		$filterTeacher = intval( mosGetParam( $_REQUEST, 'filter_teacher', $this->session->get('filter_teacher', 0)));
		$this->session->set('filter_stu', $filterStudent);
		$this->session->set('filter_teacher', $filterTeacher);

		$params = $this->container['registry']->get('params');
		$params['limit'] = $limit;
		$params['limitStart'] = $limitStart;
		$params['filterStudent'] = $filterStudent;
		$params['filterTeacher'] = $filterTeacher;
		$params['doRequery'] = $doRequery;
		$this->container['registry']->set('params', $params);


		$pluginResults = $this->dispatcher->trigger('onConferenceShowBooking', array($this->container));
		$result = $this->responseHandler->handle($pluginResults);

		$this->_render(array('content' => $result));
	}

	public function past()
	{

		if(!$this->acl->CheckPermissions('conference', 'manage')){
			throw new \LMS\Libraries\Exception\Access\Forbidden(_JLMS_ERROR_ACCESS_FORBIDDEN);
		}

		$limit = $this->input->getInt('limit', $this->session->get('list_limit', $this->config->get('list_limit')));
		$this->session->set('list_limit', $limit);
		$limitStart = $this->input->getInt('limitstart', $this->session->get('limitstart_confbook',-99));

		$doRequery = false;
		if ($limitStart == -99) {
			$limitStart = 0;
			$doRequery = true;
		}

		$this->session->set('limitstart_confbook', $limitStart);

		$filterStudent	= intval( mosGetParam( $_REQUEST, 'filter_stu', $this->session->get('filter_stu',0) ) );
		$filterTeacher = intval( mosGetParam( $_REQUEST, 'filter_teacher', $this->session->get('filter_teacher', $this->user->id)));
		$this->session->set('filter_stu', $filterStudent);
		$this->session->set('filter_teacher', $filterTeacher);

		$params = $this->container['registry']->get('params');
		$params['limit'] = $limit;
		$params['limitStart'] = $limitStart;
		$params['filterStudent'] = $filterStudent;
		$params['filterTeacher'] = $filterTeacher;
		$params['doRequery'] = $doRequery;
		$this->container['registry']->set('params', $params);

		$pluginResults = $this->dispatcher->trigger('onConferenceShowPastBooking', array($this->container));
		$result = $this->responseHandler->handle($pluginResults);

		$this->_render(array('content' => $result));
	}

	/**
	 * Method to show form for add new booking
	 */
	public function add()
	{
		if(!$this->acl->CheckPermissions('conference', 'manage')){
			throw new \LMS\Libraries\Exception\Access\Forbidden(_JLMS_ERROR_ACCESS_FORBIDDEN);
		}

		$results = $this->dispatcher->trigger( 'onConferenceNewBooking', array($this->container));
		$result = $this->responseHandler->handle($results);
		$this->_render(array('content' => $result));
	}

	/**
	 * Method to save changes to booking
	 */
	public function save()
	{
		if(!$this->acl->CheckPermissions('conference', 'manage')){
			throw new \LMS\Libraries\Exception\Access\Forbidden(_JLMS_ERROR_ACCESS_FORBIDDEN);
		}

		$params = $this->container['registry']->get('params');
		if($params['bookedConferenceID'] == 0){
			$params['weekDay'] = mosGetParam($_POST,'weekday');
			$params['monthDay'] = mosGetParam($_POST,'monthday');
			$this->container['registry']->set('params', $params);
			$results = $this->dispatcher->trigger( 'onConferenceSaveBooking', array($this->container));
		}else{
			$results = $this->dispatcher->trigger( 'onConferenceUpdateBooking', array($this->container));
		}
		$this->responseHandler->handle($results);
		$this->setRedirect(html_entity_decode(sefRelToAbs('index.php?option=com_joomla_lms&Itemdid='.$params['Itemid'].'&task=conference.booking&id='.$params['courseID'])));
	}

	/**
	 * Method to view details about booked conference
	 */
	public function view()
	{
		$bookingID = $this->input->getInt('bookedConferenceID', false);
		$params = $this->container['registry']->get('params');

		if($bookingID == false){
			$this->setRedirect(html_entity_decode(sefRelToAbs('index.php?option=com_joomla_lms&Itemdid='.$params['Itemid'].'&task=conference&id='.$params['courseID'])));
		}

		$pluginResults = $this->dispatcher->trigger('onConferenceViewBooking', array($this->container));
		$result = $this->responseHandler->handle($pluginResults);

		$this->_render(array('content' => $result));
	}

	/**
	 * Method to show form to edit existing booking
	 */
	public function edit()
	{
		if(!$this->acl->CheckPermissions('conference', 'manage')){
			throw new \LMS\Libraries\Exception\Access\Forbidden(_JLMS_ERROR_ACCESS_FORBIDDEN);
		}

		$results = $this->dispatcher->trigger( 'onConferenceEditBooking', array($this->container));
		$result = $this->responseHandler->handle($results);
		$this->_render(array('content' => $result));
	}

	/**
	 * Method to show form to edit existing booking
	 */
	public function change()
	{
		if(!$this->acl->CheckPermissions('conference', 'manage')){
			throw new \LMS\Libraries\Exception\Access\Forbidden(_JLMS_ERROR_ACCESS_FORBIDDEN);
		}

		$params = $this->container['registry']->get('params');
		$results = $this->dispatcher->trigger( 'onConferenceChangeBooking', array($this->container));
		$this->responseHandler->handle($results);
		$this->setRedirect(html_entity_decode(sefRelToAbs('index.php?option=com_joomla_lms&Itemdid='.$params['Itemid'].'&task=conference.booking&id='.$params['courseID'])));
	}

	/**
	 * Method to delete existing booking
	 */
	public function delete()
	{
		if(!$this->acl->CheckPermissions('conference', 'manage')){
			throw new \LMS\Libraries\Exception\Access\Forbidden(_JLMS_ERROR_ACCESS_FORBIDDEN);
		}

		$params = $this->container['registry']->get('params');
		$results = $this->dispatcher->trigger( 'onConferenceDeleteBooking', array($this->container));
		$this->responseHandler->handle($results);
		$this->setRedirect(html_entity_decode(sefRelToAbs('index.php?option=com_joomla_lms&Itemdid='.$params['Itemid'].'&task=conference.booking&id='.$params['courseID'])));
	}

}