<?php
defined('JPATH_PLATFORM') or die;

use LMS\Libraries;
use LMS\Libraries\ServiceLocator;

JFormHelper::loadFieldClass('list');

class JFormFieldStudent extends JFormFieldList
{	
	protected $type = 'Student';

	protected function getInput()
	{
		$html = array();
		$attr = '';

		// Initialize some field attributes.
		$attr .= !empty($this->class) ? ' class="' . $this->class . '"' : '';
		$attr .= !empty($this->size) ? ' size="' . $this->size . '"' : '';
		$attr .= $this->multiple ? ' multiple' : '';
		$attr .= $this->required ? ' required aria-required="true"' : '';
		$attr .= $this->autofocus ? ' autofocus' : '';

		// To avoid user's confusion, readonly="true" should imply disabled="true".
		if ((string) $this->readonly == '1' || (string) $this->readonly == 'true' || (string) $this->disabled == '1'|| (string) $this->disabled == 'true')
		{
			$attr .= ' disabled="disabled"';
		}

		// Initialize JavaScript field attributes.
		$attr .= $this->onchange ? ' onchange="' . $this->onchange . '"' : '';

		// Get the field options.
		$options = (array) $this->getOptions();

		if(!count($options)){
			return;
		}

		// Create a read-only list (no name) with hidden input(s) to store the value(s).
		if ((string) $this->readonly == '1' || (string) $this->readonly == 'true')
		{
			$html[] = JHtml::_('select.genericlist', $options, '', trim($attr), 'value', 'text', $this->value, $this->id);

			// E.g. form field type tag sends $this->value as array
			if ($this->multiple && is_array($this->value))
			{
				if (!count($this->value))
				{
					$this->value[] = '';
				}

				foreach ($this->value as $value)
				{
					$html[] = '<input type="hidden" name="' . $this->name . '" value="' . htmlspecialchars($value, ENT_COMPAT, 'UTF-8') . '"/>';
				}
			}
			else
			{
				$html[] = '<input type="hidden" name="' . $this->name . '" value="' . htmlspecialchars($this->value, ENT_COMPAT, 'UTF-8') . '"/>';
			}
		}
		else
			// Create a regular list.
		{
			$html[] = JHtml::_('select.genericlist', $options, $this->name, trim($attr), 'value', 'text', $this->value, $this->id);
		}

		return implode($html);
	}
	
	protected function getOptions()
	{
		$course_id = (int)\JFactory::getApplication()->getUserStateFromRequest( 'com_joomla_lms.filter.course_id', 0 );
		$status = (int)\JFactory::getApplication()->getUserStateFromRequest( 'com_joomla_lms.filter.status', 0 );
		$group_id = (int)\JFactory::getApplication()->getUserStateFromRequest( 'com_joomla_lms.filter.group_id', 0 );
		$user_id = (int)\JFactory::getApplication()->getUserStateFromRequest( 'com_joomla_lms.filter.'.$this->fieldname, 0 );

		ServiceLocator::load('tool.users');
		$studentRepository	= ServiceLocator::get('studentRepository');

		$filterOptions = array();
		$filterOptions['fields'] = 'student.profile.id, student.profile.name';
		$filterOptions['course_id'] = $course_id;
		if($group_id){
			$filterOptions['group_id'] = $group_id;
		}
		if($status){
			$filterOptions['homeworkStatus'] = $status;
		}

		$items = $studentRepository->getByFilter($filterOptions);

		$options = array();

		if(isset($items) && is_array($items) && count($items)){
			$options[] = JHtml::_('select.option', 0, _JLMS_HW_FILTER_ALL_STUDENTS, 'value', 'text', false);

			foreach($items AS $item) {
				$options[] = JHtml::_('select.option', $item->id, $item->name, 'value', 'text',	false);
			}

			$this->setValue( $user_id );
		}

		return $options;		
	}

}