<?php
namespace LMS\Tools\Mailbox\Model;
 
defined('_JEXEC') or die;

use LMS\Libraries\Repository;
use LMS\Libraries\User;
use LMS\Libraries\Base;
use LMS\Libraries\ServiceLocator;
use LMS\Tools\Mailbox\Repositories as Rep;

class Message extends Base\Model
{	
	var $modelDir = __DIR__;
	
	public function __construct($options = array())
	{		
		$this->repository = new Rep\Mailbox();
		
		$this->config = ServiceLocator::get('config');
		
		parent::__construct($options);
	}	

	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_joomla_lms.message', 'message', array('control' => 'jform', 'load_data' => $loadData));
		
		if (empty($form))
		{
			return false;
		}

		return $form;
	}

	public function loadFormData()
	{		
		$messageId = $this->getState('message_id');
		
		if(!$messageId) {
			return false;
		}
		
		$data= $this->getMessage($messageId);
		
		return $data;
	}	
	
	public function getMessage($id)
	{		
		$this->user = ServiceLocator::get('user');
		
		$mailbox = new Rep\Mailbox();
		$message = $mailbox->getByID($id);				
		
		if($message->sender_id == $this->user->id) 
		{
			$repRecipient = new Rep\Recipient();				
			$message->recipients = $repRecipient->getByFilter(array('fields' => 'recipient.user_id',  'resultType' => 'column', 'message_id' => $id));		
		} else {
			$message->recipients = array($this->user->id);
		}		
		
		if($message->file) {			
			$fileRep	= ServiceLocator::get('file_rep');
			$fileData = $fileRep->getById($message->file);
			
			$message->file_name = $fileData->file_name;
		}		
		
		return $message;
	}	
	
	public function sendMail($data) 
	{		
		$joomlaCfg = ServiceLocator::get('joomla_config');
		
		$form = $this->getForm('message');
				
		$data = $form->filter($data);
		$form->validate($data);		
		$form->bind($data);		
				
		$subject		= $data['subject'];		
		$body			= $data['message'];				
						
		$recipients	= $form->getField('recipients')->value;
		$parentId	= $form->getField('parent_id')->value;
				
		$courseId = $this->config->get('course_id');
		
		$filter = \JFilterInput::getInstance();	
		
		$fileId = 0;
				
		if(isset($_FILES['jform']) && $_FILES['jform']['name']['file'])
		{			
			$fileId = \JLMS_uploadFile( $courseId, array('tmp_name' => $_FILES['jform']['tmp_name']['file'], 'name' => $_FILES['jform']['name']['file']));			
		}
		
		$filePath = '';
		$filename = '';
		
		if($fileId)
		{
			$fileRep	= ServiceLocator::get('file_rep');		
			$fileData = $fileRep->getById($fileId);			
			
			if ($fileData)
			{
				$file_path = _JOOMLMS_DOC_FOLDER.'/'.$fileData->file_srv_name;
				$filename = $fileData->file_name;
			}
		}
				
		$this->repository->save(array(			
			'sender_id' => $this->getState('userId'),
			'course_id' => $courseId,
			'subject' => $subject,
			'date' => \gmdate("Y-m-d H:i:s"),
			'message' => $body,
			'file' => $fileId,			
			'del' => 0,
			'parent_id' => $parentId
		));
		$mailId = $this->repository->getId();				
		
		$mailCfgRep = new Rep\MailboxConfig();	
		$mailTitle = $mailCfgRep->get('mail_title');
		$mailBody = $mailCfgRep->get('mail_body');
		
		if(empty($mailTitle)){
			$mailTitle = "{sitename}: New message notification";
		}							
		$mailTitle = str_replace('{sitename}', $this->config->get('sitename'), $mailTitle);
		
		if(empty($mailBody)){
			$mailBody = "Hello,
				{name} has just sent new message '{title}'.
				This message is located at {link}
				
				This is the message that was sent:
				***************
				{message}
				***************
				Please do not reply to this email. "
				;
		}
		
		$link  = $this->config->get('live_site').'/index.php?option=com_joomla_lms&amp;Itemid='.$this->config->get('Itemid').'&amp;task=mailbox.message.show&amp;course_id='.$courseId.'&amp;message_id='.$mailId;
				
		$mailBody = nl2br(stripslashes($mailBody));
		$mailBody = strtr($mailBody, array(
			'{sitename}'=>  $this->config->get('sitename'),
			'{name}'	=>  $this->getState('userName'),
			'{email}'	=>  $this->getState('userEmail'),
			'{title}'	=>  $subject,
			'{message}'	=>  $body,
			'{link}'	=>  '<a href="'.$link.'">'.$link.'</a>'
		));
		
		$recipientRep	= new Rep\Recipient();	
		$userRep	= ServiceLocator::get('user_rep');	
		
		$userData = $userRep->getByFilter(array(
			'fields' => 'joomlauser.id, joomlauser.email, joomlauser.name',
			'user_id' => $recipients			
		));		
		
		$data = array();
		foreach($userData AS $row) {
			$data[] = array(				
				'message_id' => $mailId,
				'user_id' => $row->id
			);
		}
		
		$recipientRep->saveMultiple($data);		
											
		if($this->config->get('mess_enotify')) {
			$mailer = ServiceLocator::get('mailer');
			$mailer->setSender(array($joomlaCfg->get('mailfrom'), $joomlaCfg->get('fromname')))
				->setSubject($mailTitle)
				->setBody($mailBody)
				->isHtml(true);
				
			foreach($userData AS $row) {
					$mailer->ClearAddresses();
					$mailer->addRecipient($row->email);
					$mailer->send();
			}		
		}	
	}

	public function loadFile($messageId, $userId) 
	{		
		$file = $this->repository->getByFilter(array(
			'fields' => 'message.file',
			'where'=>'message.id = '.$messageId.' AND (message.sender_id = '.$userId.' OR message.recipient.user_id = '.$userId.')',
			'resultType' => 'result'
		));		
		
		if($file){
			\JLMS_downloadFile($file);
		}	
	}
	
	public function markAsRead($id, $userId)
	{			
		$repRecipient = new Rep\Recipient();
		$repRecipient->update(array('is_read'=>1), array('message_id'=>$id, 'user_id'=>$userId));		
	}	
	
	public function addHistoryToBody($body, $date, $replay = false)
	{		
		$mainData = explode('-',substr($date,0,10));
		$otherData = explode(':',substr($date,11,8));
		
		$viewFormat = JLMS_dateToDisplay(date(\JText::_('_JLMS_MB_REPL_DF'), mktime($otherData[0],$otherData[1],$otherData[2],$mainData[1],$mainData[2],$mainData[0])));
		
		$text = $viewFormat.' '.($replay?'':\JText::_('_JLMS_MB_REPL_YW').':').'<br />';
		$linebreak = '<br /><br />';
		$str = '';
		for($i=0;$i<50;$i++){							
			$str = $str.'-';
		}
		$linebreak .= $str;
		$linebreak .= '<br />';
		
		$body = $linebreak.$text.$body;
		
		return $body;
		
	}
}
