<?php
 
namespace LMS\Tools\Users\Controllers;

defined('_JEXEC') or die;

use LMS\Libraries\Base;

DEFINE('JLMS_LIST_LIMIT', 5);
 
class Users extends Base\Controller
{
	public function __construct($config = array())
	{	
		parent::__construct($config);
	}
	
	public function display($cachable = false, $urlparams = false)
	{
		$courseId = $this->config->get('course_id');
		$itemId = $this->config->get('itemId');		
		
		if(!$this->user->get('id')) {
			$this->setRedirect(sefRelToAbs('index.php?option=com_joomla_lms&id='.$courseId));
			return JError::raiseError(401, JText::_('You are not authorized to view this resource'));
		}
		
		$model	= $this->getModel('users');
		$format = $this->input->get('format','html');		
		$view	= $this->getView('users', $format);	
		$view->def('courseId', $courseId);
		$view->template = $this->app->getTemplate();
		$view->setModel($model, true);		
		$view->userlist();
	}	
	
	public function studentsListModal()
	{	
		$courseId = $this->config->get('course_id');
		$itemId = $this->config->get('itemId');
		$context = $this->input->get('context','');		
		
		if(!$this->user->get('id')) {
			$this->setRedirect(sefRelToAbs('index.php?option=com_joomla_lms&id='.$courseId));
			return JError::raiseError(401, JText::_('You are not authorized to view this resource'));
		}
		
		if(!empty($context)) {
			switch($context){
				case 'students':
					$this->studentsTableAjax();
				break;
				case 'staff':
					$this->staffTableAjax();
				break;
				case 'groups':
					$this->groupsTableAjax();
				break;				
			}
		} else {
			/*students table block*/
			$model = $this->getModel('students');
			
			$this->populateModel($model);
			
			$filters = $model->getState('list.filters');		
			$filters['search'] = ''; //reset search filter
			$filters = $model->setState('list.filters', $filters);
			
			$model->setState('course_id', $courseId);
			$model->setState('list.start', 0);
			$model->setState('list.limit', JLMS_LIST_LIMIT);
			/*students table block*/		
			
			/*staff table block*/
			$modelStaff = $this->getModel('staff');
			
			$this->populateModel($modelStaff);								
			
			$filters = $modelStaff->getState('list.filters');		
			$filters['search'] = ''; //reset search filter
			$filters = $modelStaff->setState('list.filters', $filters);
						
			$modelStaff->setState('course_id', $courseId);		
			$modelStaff->setState('list.start', 0);
			$modelStaff->setState('list.limit', JLMS_LIST_LIMIT);					
			/*staff table block*/
			
			/*groups table block*/
			$modelGroups = $this->getModel('groups');			
			
			$this->populateModel($modelGroups);
			
			$filters = $modelGroups->getState('list.filters');		
			$filters['search'] = ''; //reset search filter
			$filters = $modelGroups->setState('list.filters', $filters);
			
			$modelGroups->setState('course_id', $courseId);		
			$modelGroups->setState('list.start', 0);
			$modelGroups->setState('list.limit', JLMS_LIST_LIMIT);		
			/*groups table block*/
			
			$view	= $this->getView('users', 'html');
			$view->def('courseId', $courseId);
			$view->template = $this->application->getTemplate();									
			$view->setModel($model, true);
			$view->setModel($modelStaff);
			$view->setModel($modelGroups);			
			$view->studentsListModal();
		}
		
		die();
	}
	
	public function studentsTableAjax() 
	{		
		$courseId = $this->config->get('course_id');
		$itemId = $this->config->get('itemId');				
		
		$model = $this->getModel('students');
							
		$this->populateModel($model);	
		
		$filters = $model->getState('list.filters');
		$filters['search'] = isset($filters['search'])?$filters['search']:'';		
		if(!empty($filters['usergroup'])) {
			$model->setState('group_id', $filters['usergroup']);
		}
				
		$model->setState('course_id', $courseId);
		$model->setState('list.limit', JLMS_LIST_LIMIT);
		
		$view = $this->getView('users', 'html');
		$view->def('courseId', $courseId);
		$view->template = $this->application->getTemplate();		
		$view->setModel($model, true);
		
		echo $view->studentsTable(true);
		
		die();
	}
	
	public function staffTableAjax() 
	{
		$courseId = $this->config->get('course_id');
		$itemId = $this->config->get('itemId');		
		
		$model = $this->getModel('staff');
			
		$this->populateModel($model);
		
		$filters = $model->getState('list.filters');		
		$filters['search'] = isset($filters['staff_search'])?$filters['staff_search']:'';
		$model->setState('list.filters', $filters);
				
		$model->setState('course_id', $courseId);				
		$model->setState('list.limit', JLMS_LIST_LIMIT);		
		
		$view	= $this->getView('users', 'html');
		$view->def('courseId', $courseId);			
		$view->template = $this->application->getTemplate();		
		$view->setModel($model, true);
	
		echo $view->staffTable(true);
		
		die();
	}
	
	public function groupsTableAjax() 
	{
		$courseId = $this->config->get('course_id');
		$itemId = $this->config->get('itemId');		
		
		$model = $this->getModel('groups');
			
		$this->populateModel($model);
		
		$filters = $model->getState('list.filters');		
		$filters['search'] = isset($filters['groups_search'])?$filters['groups_search']:'';
		$model->setState('list.filters', $filters);		
			
		$model->setState('course_id', $courseId);				
		$model->setState('list.limit', JLMS_LIST_LIMIT);
		
		$view	= $this->getView('users', 'html');
		$view->def('courseId', $courseId);			
		$view->template = $this->application->getTemplate();		
		$view->setModel($model, true);
		
		echo $view->groupsTable(true);
				
		die();
	}
	
	public function jsonCountStudentsInGroup()
	{	
		$courseId = $this->config->get('course_id');
		$itemId = $this->config->get('itemId');		
		
		if(!$this->user->get('id')) {
			$this->setRedirect(sefRelToAbs('index.php?option=com_joomla_lms&id='.$courseId));
			return JError::raiseError(401, JText::_('You are not authorized to view this resource'));
		}
		
		$model	= $this->getModel('students');		
		$count = $model->countStudentsInGroup(
			$this->input->get('groups', array(-1), 'array'), 
			$this->input->get('selectedStudents', array(), 'array'),
			$this->input->get('exceptedStudents', array(), 'array')			
		);
		
		echo json_encode(array('count'=>$count));
		
		die();
	}
	
	public function jsonNamesData()
	{	
		$courseId = $this->config->get('course_id');
		$itemId = $this->config->get('itemId');		
		
		if(!$this->user->get('id')) {
			$this->setRedirect(sefRelToAbs('index.php?option=com_joomla_lms&id='.$courseId));
			return JError::raiseError(401, JText::_('You are not authorized to view this resource'));
		}
		
		$chosenStudents = $this->input->get('students', array(), 'array');
		$chosenGroups = $this->input->get('groups', array(), 'array');
		$chosenStaff = $this->input->get('staff', array(), 'array');		
		$excludedStudents = $this->input->get('excludedStudents', array(), 'array');
		$excludedGroups = $this->input->get('excludedGroups', array(), 'array');
		$excludedStaff = $this->input->get('excludedStaff', array(), 'array');
		
		if(in_array('*', $chosenStudents) && empty($excludedStudents)) {
			$students = \JText::_('_JLMS_ALL');
		} else {
			$modelStudents = $this->getModel('students');
			if(in_array('*', $chosenStudents)) {
				$chosenStudents = array();
			} 
			
			$students = $modelStudents->getStudentsNames($chosenStudents, $excludedStudents);					
		} 
		
		if(in_array('*', $chosenStaff) && empty($excludedStaff)) {
			$staff = \JText::_('_JLMS_ALL');
		} else {
			$modelStaff = $this->getModel('staff');
			if(in_array('*', $chosenStaff)) {
				$chosenStaff = array();
			} 
			
			$staff = $modelStaff->getStaffNames($chosenStaff, $excludedStaff);			
		}

		if(in_array('*', $chosenGroups) && empty($excludedGroups)) {
			$groups = \JText::_('_JLMS_ALL');
		} else {
			$modelGroups = $this->getModel('groups');
			if(in_array('*', $chosenGroups)) {
				$chosenGroups = array();
			} 
			
			$groups = $modelGroups->getGroupsNames($chosenGroups, $excludedGroups);			
		}		
		
				
		$data = array();
		$data['groups'] = $groups;
		$data['staff'] = $staff;
		$data['students'] = $students;
		
		echo json_encode($data);
		
		die();
	}
}