<?php
/**
 * @package     Joomla.Site
 * @subpackage  com_joomla_lms
 *
 */

namespace LMS\Widgets;

use LMS\Libraries;
use LMS\Libraries\Base;

class ContentToolbar extends Base\Widget
{	
	protected $buttons = array();
	
	protected $content = '';
	
	protected $glueButtons = false;
	
	protected $buttonSize = '';
	
	protected $hideButtonTitle = false;

	public function __construct($params = array())
	{
		parent::__construct($params);	
	}

	/**
	 * @param array $params
	 * @return \LMS\Widgets\Button
	 */
	public function addCustomButton($params = array()) 
	{		
		$params['glue'] = $this->glueButtons;		
		$params['size'] = $this->buttonSize;
		$params['showTitle'] = !$this->hideButtonTitle;
		
		if(!isset($params['type'])) {
			$params['type'] = 'custom';
		}
				
		$alias = base64_encode(time().rand(1, 100000).rand(1, 100000).rand(1, 100000));
		
		$button = $this->widget('button', $params, $alias);		
		
		$this->buttons[] = $button;		
		
		return $button;
	}
	
	public function addActionsButton($params = array()) 
	{
		$params['type'] = 'dropdown';		
		$params['icon'] = 'wrench';		
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_ACTIONS;
		}
		
		return $this->addCustomButton($params);
	}
	
	public function addImportButton($params = array()) 
	{		
		$params['icon'] = 'download';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_IMPORT;
		}
		
		return $this->addCustomButton($params);
	}
	
	public function addExportButton($params = array()) 
	{		
		$params['icon'] = 'upload';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_EXPORT;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPrintButton($params = array()) 
	{		
		$params['icon'] = 'print';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_PRINT_RESULTS;
		}
		
		return $this->addCustomButton($params);
	}

	public function addEmailButton($params = array()) 
	{		
		$params['icon'] = 'mail';
		
		if(!isset($params['title'])) {
			$params['title'] = 'E-mail';//replace with JText!... at the moment title is passed through params
		}
		
		return $this->addCustomButton($params);
	}

	public function addCertificateButton($params = array()) 
	{		
		$params['icon'] = 'image';
		
		if(!isset($params['title'])) {
			$params['title'] = 'Certificate';//replace with JText!... at the moment title is passed through params
		}
		
		return $this->addCustomButton($params);
	}

	public function addModalButton($params = array()) 
	{			
		\JHtml::_('behavior.modal', 'a.modal');
		
		$params['type'] = 'modal';		
		$params['icon'] = 'out-2';

		if(!isset($params['title'])) {
			$params['title'] = _JLMS_MODAL;
		}
		
		return $this->addCustomButton($params);				
	}
	
	public function addBackButton($params = array()) 
	{					
		$params['icon'] = 'arrow-left';

		if(!isset($params['title'])) {
			$params['title'] = _JLMS_BACK;
		}
		
		if(!isset($params['click']) && !isset($params['href'])) {
			$params['click'] = 'window.history.go(-1);';
		}
		
		return $this->addCustomButton($params);		
	}
	
	public function addCancelButton($params = array()) 
	{			
		$params['icon'] = 'cancel';
		$params['class'] = 'btn-cancel';

		if(!isset($params['title'])) {
			$params['title'] = _JLMS_CANCEL;
		}		
		
		return $this->addCustomButton($params);		
	}

	public function addCloseButton($params = array()) 
	{		
		$params['icon'] = 'unpublish';

		if(!isset($params['title'])) {
			$params['title'] = _JLMS_CLOSE;
		}		
		
		return $this->addCustomButton($params);		
	}

	public function addEnrollButton($params = array()) 
	{		
		$params['icon'] = 'publish';
		$params['class'] = 'btn-primary';

		if(!isset($params['title'])) {
			$params['title'] = _JLMS_SUBSCRIBE;
		}		
		
		return $this->addCustomButton($params);		
	}

	public function addAddToCartButton($params = array()) 
	{		
		$params['icon'] = 'publish';
		$params['class'] = 'btn-primary';
		//$params['jlmsicon'] = 'cart';

		if(!isset($params['title'])) {
			$params['title'] = _JLMS_ADD_TO_CART;
		}		
		
		return $this->addCustomButton($params);		
	}

	public function addViewCartButton($params = array()) 
	{		
		$params['icon'] = 'publish';
		$params['jlmsicon'] = 'cart';
		//$params['class'] = 'btn-primary';

		if(!isset($params['title'])) {
			$params['title'] = _JLMS_VIEW_CART;
		}		
		
		return $this->addCustomButton($params);		
	}

	public function addBuyButton($params = array()) 
	{		
		$params['icon'] = 'publish';
		$params['jlmsicon'] = 'subscriptions';
		$params['class'] = 'btn-primary';

		if(!isset($params['title'])) {
			$params['title'] = _JLMS_CHECKOUT_ITEMS;
		}		
		
		return $this->addCustomButton($params);		
	}

	public function addToggleButton($params = array())
	{
		if(isset($params['statesassoc']) && isset($params['state'])) {
			if(in_array($params['state'], array_keys($params['statesassoc']))) {				
				$params = array_merge($params, $params['statesassoc'][$params['state']]);
			} 
		}

		if(!isset($params['type'])) {
			$params['type'] = 'custom';
		}
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_PUBLISH;
		}
		
		return $this->addCustomButton($params);
	}
	
	public function addTogglePublishButton($params = array())
	{
		if(!isset($params['statesassoc'])) {
			$params['statesassoc'] = array(
				0 => array('icon' => 'publish', 'title' => _JLMS_UNPUBLISH),
				1 => array('icon' => 'unpublish', 'title' => _JLMS_PUBLISH)
			);
		}
		
		return $this->addToggleButton($params);		
	}
	
	public function addNewButton($params = array()) 
	{			
		$params['type'] = 'button';
		$params['icon'] = 'plus';
		$params['class'] = 'btn-primary';
		
		if(isset($params['href'])) {
			$params['type'] = 'custom';
		}
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_NEW;
		}		
		
		return $this->addCustomButton($params);
	}
	
	public function addSaveButton($params = array()) 
	{			
		$params['type'] = 'submit';
		$params['icon'] = 'save';
		$params['class'] = 'btn-success';
		
		if(isset($params['href'])) {
			$params['type'] = 'custom';
		}
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_SAVE;
		}	
		
		return $this->addCustomButton($params);		
	}

	public function addApplyButton($params = array()) 
	{			
		$params['type'] = 'submit';
		$params['icon'] = 'save';
		
		if(isset($params['href'])) {
			$params['type'] = 'custom';
		}
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_APPLY;
		}	
		
		return $this->addCustomButton($params);		
	}
	
	public function addEditButton($params = array()) 
	{			
		$params['icon'] = 'pencil';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_EDIT;
		}		
		
		return $this->addCustomButton($params);		
	}
	
	public function addDeleteButton($params = array()) 
	{			
		$params['icon'] = 'trash';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_DELETE;
		}		
		
		return $this->addCustomButton($params);		
	}

	/**
	 * @param array $params
	 * @return Button
	 */
	public function addDropDownButton($params = array()) 
	{	
		$params['type'] = 'dropdown';
		
		return $this->addCustomButton($params);
	}

	public function addPublishButton($params = array()) 
	{			
		$params['icon'] = 'publish';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_PUBLISH;
		}
		
		return $this->addCustomButton($params);
	}
	
	public function addUnpublishButton($params = array()) 
	{			
		$params['icon'] = 'unpublish';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_UNPUBLISH;
		}
		
		return $this->addCustomButton($params);		
	}
	
	public function addCopyButton($params = array()) 
	{		
		$params['icon'] = 'copy';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_COPY;
		}
		
		return $this->addCustomButton($params);
	}
	
	public function addMoveButton($params = array()) 
	{		
		$params['icon'] = 'move';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_MOVE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPreviewButton($params = array()) 
	{		
		$params['icon'] = 'search';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_PREVIEW_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayStartButton($params = array()) 
	{		
		$params['icon'] = 'play';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_START_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayRestartButton($params = array()) 
	{		
		$params['icon'] = 'loop';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_RESTART_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayContinueButton($params = array()) 
	{		
		$params['icon'] = 'play';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_CONTINUE_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayRetryButton($params = array()) 
	{		
		$params['icon'] = 'loop';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = \JText::_('_JLMS_RETRY');
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayResumeButton($params = array()) 
	{		
		$params['icon'] = 'play';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_RESUME_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayNextButton($params = array()) 
	{		
		$params['icon'] = 'next';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_NEXT_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayPrevButton($params = array()) 
	{		
		$params['icon'] = 'previous';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_PREV_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlaySkipButton($params = array()) 
	{		
		$params['icon'] = 'refresh';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_SKIP_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayOKButton($params = array()) 
	{		
		$params['icon'] = 'ok';
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_OK_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}

	public function addPlayContentsButton($params = array()) 
	{		
		$params['icon'] = 'stack';
		
		if(!isset($params['title'])) {
			$params['title'] = _JLMS_CONTENTS_ALT_TITLE;
		}
		
		return $this->addCustomButton($params);
	}
	
	public function addSeparator($params = array()) 
	{	
		$params['type'] = 'separator';
		
		$this->addCustomButton($params);		
	}
	
	public function addInfoButton($params = array()) 
	{			
		$params['icon'] = 'info';
		$params['active'] = true;
		
		$this->addCustomButton($params);		
	}
	
	public function addSaveNextButton($params = array()) 
	{
		$params['class'] = 'btn-success';
		
		if(!isset($params['title'])) {
			$params['title'] = \JText::_('_JLMS_SAVE_NEXT');
		}
		
		if(!isset($params['iconPosition'])) {
			$params['iconPosition'] = 'right';
		}
		
		if(!isset($params['icon'])) {
			$params['icon'] = 'arrow-right';
		}
		
		$this->addCustomButton($params);
	}
}